@@ -102,3 +102,40 @@ position_matrix <- function(i, j, shifts=0, count_once=T, distance_as_value=F, a
   mat = methods::as(mat, 'dgTMatrix')
   mat[return_i,,drop=F]
 }
+
+
+globalFeatureVector <- R6::R6Class("globalFeatureVector",
+                                   ## very restricted sparse vector class for quick indices lookup of features based on global_i (with windows between contexts)
+                                   public = list(.v = NULL,
+                                                 .levels = NULL,
+                                                 initialize = function(feature, global_i, empty_str=''){
+                                                   feature = fast_factor(feature)
+                                                   self$.levels = c(empty_str, levels(feature))
+                                                   self$.v = Matrix::sparseVector(as.numeric(feature), global_i, max(global_i))
+                                                 },
+                                                 get = function(i, ignore_empty=F, allow_na=F) {
+                                                   if (allow_na){
+                                                     is_na = i < 0 | i > length(self$.v)
+                                                     i = i[!is_na]
+                                                   }
+                                                   if(ignore_empty) {
+                                                     allow_na = F
+                                                     v = self$.v[i]
+                                                     if (!allow_na) return(self$.levels[v[v > 0] + 1]) else v_notna = self$.levels[v[v > 0] + 1]
+                                                   } else {
+                                                     if (!allow_na) return(self$.levels[self$.v[i] + 1]) else v_notna = self$.levels[self$.v[i] + 1]
+                                                   }
+                                                   v = rep(NA, length(is_na))
+                                                   v[!is_na] = v_notna
+                                                   return(v)
+                                                 }
+                                   )
+)
+
+#' @export
+'[.globalFeatureVector' = function(x,i, ignore_empty=F, allow_na=F) {
+  if (methods::is(i, 'logical')) x$get(which(i)) else x$get(i, ignore_empty=ignore_empty, allow_na=allow_na)
+}
+
+#' @export
+length.globalFeatureVector = function(x) length(x$.v)
