@@ -25,29 +25,6 @@ tokenize_to_dataframe <- function(x, doc_id=1:length(x), split_sentences=F, max_
   tokens
 }
 
-custom_dot_abbreviations <- function(){
-  ## stringi::stri_split_boundaries can handle many cases where a dot is not used at the end of a sentence, but some language specific abbreviations have to be added manually
-  ## this is mostly important for honorifics, because then the token after the dot abbreviations is often a name, and thus capitalized
-  ## here we can add abbreviations. Note that all abbreviations are matched with case-insensitive regex, and are assumed to be followed by a dot
-  ## (there should be a list somewhere for this stuff, right?)
-  common_titles = c('mr','miss','mrs','ms','mx',  ## english
-                    'dhr', 'mw', 'mevr',          ## dutch
-                    'herr', 'frau')               ## german
-  prof_titles = c('prof', 'dr', 'drs')
-
-  c(common_titles, prof_titles)
-}
-
-escape_custom_dot_abbreviations <- function(x){
-  cda = custom_dot_abbreviations()
-  r = stringi::stri_paste(cda, collapse = '|')
-  stringi::stri_replace_all(x, replacement = '___CuDoAb___', regex=sprintf('\\b(?<=(%s))\\.', r), case_insensitive=T)
-}
-
-unescape_custom_dot_abbreviation <- function(x) {
-  stringi::stri_replace_all(x, '.', fixed = '___CuDoAb___')
-}
-
 split_tokens <- function(x, max_tokens, remember_spaces=F) {
   x = stringi::stri_split_boundaries(x, type='word')
   if (remember_spaces) {
@@ -64,10 +41,10 @@ tokenize_to_dataframe_batch <- function(x, doc_id, split_sentences=F, max_senten
   x = gsub('_', ' ', x, fixed=T)
 
   if (split_sentences | !is.null(max_sentences)) {
-    x = escape_custom_dot_abbreviations(x)
     x = stringi::stri_split_boundaries(x, type='sentence')
 
     if (!is.null(max_sentences)) x = sapply(x, head, max_sentences)
+
     x = lapply(x, function(x) unlist_to_df(split_tokens(x, max_tokens, remember_spaces),
                                            global_position=T))
     doclen = sapply(x, function(x) length(x$id))
@@ -78,7 +55,6 @@ tokenize_to_dataframe_batch <- function(x, doc_id, split_sentences=F, max_senten
     data.table::setcolorder(x, c('doc_id','sentence','token_id','token'))
 
     if (!is.null(max_tokens)) x = x[x$position <= max_tokens,]
-    x[, token := unescape_custom_dot_abbreviation(x$token)]
   } else {
     x = split_tokens(x, max_tokens, remember_spaces)
     x = as.data.table(unlist_to_df(x, doc_id))
@@ -91,6 +67,7 @@ tokenize_to_dataframe_batch <- function(x, doc_id, split_sentences=F, max_senten
 
 unlist_to_df <- function(l, ids=1:length(l), global_position=F){
   len = sapply(l, length)
+  if (sum(len) == 0) return(NULL)
   filter = len > 0
   if (global_position){
     position = 1:sum(len)
